#include <stdio.h>
#include "driver/d_Display_DMA2D.h"
#include "driver/d_timer.h"
#include "fatfs/ff.h"
#include "jpeglib.h"
#include "mymalloc.h"


extern tImage ebssystart_logo_35;
extern tImage twoInone_display_logo;

/***********************************************************
 * Public Functions
 ***********************************************************/
 
/*****************************************************************
 *
 * Function:  void display_init(void)
 *
 * Summary:   Initializes the display
 *
 * Parameters:
 *
 * Return:
 *
 * Revision History:
 * Creation Date: 08042019
 *
 *****************************************************************/
void display_init(void) {

	SDRAM_FLASH_init();
	DMA2D_Display_init();

	switch_buffer(Layer_1, Buffer_1);

	uint16_t image_width = ebssystart_logo_35.width;
	uint16_t image_height = ebssystart_logo_35.height;
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)ebssystart_logo_35.data, CM_ARGB8888, Layer_1, Buffer_2);
	switch_buffer(Layer_1, Buffer_2);

	delay_ms(4000);

	image_width = twoInone_display_logo.width;
	image_height = twoInone_display_logo.height;
	DMA2D_Draw_Image((HDP-image_width)/2, (VDP-image_height)/2, image_width, image_height, 0xFF, REPLACE_ALPHA_VALUE, (uint32_t)twoInone_display_logo.data, CM_ARGB8888, Layer_1, Buffer_1);
	switch_buffer(Layer_1, Buffer_1);

}

/*****************************************************************
 *
 * Function:  int paint_JPEG_file(char* filename)
 *
 * Summary:   Get the JPEG with name filename from SD-card, decode it and draw it on the display
 *
 * Parameters:
 * filename - full filepath of the JPEG to be drawn
 *
 * Return:	0 - if successful, 1 - if connection to SD-card failed, 2 - if JPEG is too large
 *
 * Revision History:
 * Creation Date: 24052021
 *
 *****************************************************************/
int paint_JPEG_file (char * filename)
{
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;

  FIL infile;		// source file
  FATFS sd_fs;
  JSAMPARRAY buffer;		// Output row buffer
  int row_stride;		// physical row width in output buffer
  tTimer refTime = timer_get_time_ms();
  FRESULT fres = f_mount(&sd_fs, "0", 1);
  if(fres != FR_OK){
	  return 1;
  }
  fres = f_open(&infile, filename, FA_READ);
  if (fres != FR_OK) {
    return 1;
  }

  cinfo.err = jpeg_std_error(&jerr);

  jpeg_create_decompress(&cinfo);

  jpeg_stdio_src(&cinfo, &infile);

  jpeg_read_header(&cinfo, TRUE);

  // check if downscaling is needed. If so, set the parameter for decompression accordingly
  if(cinfo.image_width > HDP || cinfo.image_height > VDP){
	  float fract;
	  float width_fract = ((float)HDP)/cinfo.image_width;
	  float height_fract = ((float)VDP)/cinfo.image_height;
	  if(width_fract < height_fract){
		  fract = width_fract;
	  }
	  else{
		  fract = height_fract;
	  }

	  //only supported scaling ratios are M/8 with all M from 1 to 16
	  if(fract >= 7.0/8){
		  cinfo.scale_num = 7;
	  }
	  else if(fract >= 6.0/8){
		  cinfo.scale_num = 6;
	  }
	  else if(fract >= 5.0/8){
		  cinfo.scale_num = 5;
	  }
	  else if(fract >= 4.0/8){
		  cinfo.scale_num = 4;
	  }
	  else if(fract >= 3.0/8){
		  cinfo.scale_num = 3;
	  }
	  else if(fract >= 2.0/8){
		  cinfo.scale_num = 2;
	  }
	  else if(fract >= 1.0/8){
		  cinfo.scale_num = 1;
	  }
	  else{	// jpeg is too large to be shown on this display
		  return 2;
	  }
	  cinfo.scale_denom = 8;
  }

  jpeg_start_decompress(&cinfo);

  row_stride = cinfo.output_width * cinfo.output_components;

  buffer = (*cinfo.mem->alloc_sarray)
		((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

  int rgb_ctr;
  int pixel_ctr = 0;
  uint32_t pixel = 0;
  uint32_t image_data_addr = 0xC05DC000;
  while (cinfo.output_scanline < cinfo.output_height) {
    jpeg_read_scanlines(&cinfo, buffer, 1);
    rgb_ctr = 0;
    for(int i=0; i<row_stride; i++){
    	uint32_t temp = (uint32_t)*(*buffer+i);
    	switch(rgb_ctr){
    	case 0: pixel = temp<<16;
    			rgb_ctr++;
    			break;
    	case 1: pixel |= temp<<8;
    			rgb_ctr++;
    			break;
    	case 2: pixel |= temp;
    			SDRAM_Write_32b((image_data_addr - 0xC0000000)+pixel_ctr*4, pixel);	//4 byte per pixel...
    			// note that SDRAM_Write_xxx and SDRAM_Read_xxx use the offset to SDRAM_START_ADDR(=0xC0000000) as parameter
    			pixel_ctr++;
    			rgb_ctr = 0;
    			break;
    	}

    }
  }

  jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  f_close(&infile);
  f_mount(0, "0", 1);

  while(timer_get_time_ms()-refTime < 7000){}
  DMA2D_Fill_Color(0xFFFFFFFF, Layer_1, Buffer_1);
  DMA2D_Draw_Image((HDP-cinfo.output_width)/2, (VDP-cinfo.output_height)/2, cinfo.output_width, cinfo.output_height,
		  0xFF, REPLACE_ALPHA_VALUE, (uint32_t)image_data_addr, CM_ARGB8888, Layer_1, Buffer_1);

  return 0;
}



